# formr survey framework

**chain simple forms / surveys into longer runs using the power of R to generate pretty feedback and complex designs**

This is a framework that allows you to create simple and complex studies using spreadsheets with items for the surveys and "runs" for chaining together various modules.

## Runs and their modules

Simple surveys can be turned into elaborate studies by using "runs".

Runs allow you to chain various modules and thus:

* manage access and eligibility for a study
* use different pathways for different users using branches
* send e-mail invites and reminders
* implement delays/pauses
* add external modules
* loop surveys and thus enable diaries, experience-sampling studies using branches
* give feedback based on user input, using [knitr](http://yihui.name/knitr/) and [Markdown](http://daringfireball.net/projects/markdown/) through [OpenCPU](https://public.opencpu.org/pages/)'s [R](http://www.r-project.org/) API

Using branching and pauses, the following designs should be possible:

* simple one-shot surveys
* surveys with eligibility rules (using branch conditions, e.g. `demographics.age > 30`)
* diary studies (using branch conditions `COUNT(diary.session_id) < 14` -> start diary again the next day -> if finished continue)
* longitudinal studies using pauses (ie. wait 2 months after last participation). The items of wave 2 need not be finalised at wave 1.

### Pause
This simple component allows you to delay the continuation of the run until a certain date, time of day or to wait relative to a date that a user specified (such as her graduation date or the last time he cut his nails). See the **OpenCPU + R + Knitr + Markdown** section to find out how to personalise the text shown while waiting.

### Branch
Branches are components that allow to execute readonly SQL commands on all surveys you created. Depending on whether the results evaluates to true/1 or false/0, you can go to different positions in the run - these can be later in the run or earlier in the run (thus creating loops).

### TimeBranch
These components are the bastard children of a Pause + Branch: If the user accesses the run within the specified time frame (like a Pause), the run jumps to one position in the run (like a Branch). If she doesn't, the run progresses to a different position in the run (e.g. to a reminder email). This component is useful, if you need to set up a period during which a survey is accessible or if you want to send reminders automatically after some time elapsed. 
See the **OpenCPU + R + Knitr + Markdown** section to find out how to customise the text shown while waiting.

### Email
Using an SMTP gateway that you can set up in the admin area, you can send emails to your users. Using the tag `{{login_link}}`, you can send users a personalised link to the run, using the tag `{{login_code}}` you can build links for informants to report on users. See the **OpenCPU + R + Knitr + Markdown** section to find out how to personalise email text.

### External link
These are simple external links - you can use them to send users to other, specialised data collection modules, such as a social network generator. If you insert the placeholder `%s`, it will be replaced by the users run_session code, allowing you to link data later. You can choose to "end" this component before the user is redirected to the link or by enabling your external module to call our API to close it, when it's done. 

### Page
Simple text pages. See the **OpenCPU + R + Knitr + Markdown** section to find out how to generate personalised feedback.

### Survey
Surveys are series of questions that are created using simple spreadsheets/**item tables** (ie. Excel, OpenOffice, etc.: *.xls, *.ods, *.xlsx, *.csv etc.).
Survey results are stored in MySQL tables of the same name, which can be used for various conditions later on.

You can add the same survey several times or even loop using branches.  
You can also use the same survey across different runs. For example this would allow you to ask respondents for their demographic information only once. You'd do this by using a Branch with the condition `COUNT(demographics.session_id) > 0` and skipping over the demographics survey, if true.

Survey names may only contain the characters `a-zA-Z0-9_` and need to start with a letter.

#### Item table criteria

* The format must be one of .csv, .xls, .xlsx, .ods (OpenOffice), .xml, .txt
* You can use two sheets
* The first sheet should have the name **survey**, if no such sheet exists, we use the first one.
	* The first line has to contain the column names you used. Name and type should be leftmost.
	* The following column names are used. You can add others, they will be ignored. 
		* **_name_** (mandatory). This can only contain `a-zA-Z0-9_` and needs to start with a letter.
		* **_type_** (mandatory). See below.
		* **label**. You can use [Markdown](http://daringfireball.net/projects/markdown/) and HTML in the question texts. You can also use [Font Awesome](http://fontawesome.io) icons.
		* **skipif** You can refer to the same survey here or you can reference other surveys using `survey_name.variable_name`. SQL syntax.
		* **optional** You can make an item optional (most items are mandatory by default), by using the `*` character in the optional-column. Items optional by default (`check`, `btncheck`, `mmc`) can be made mandatory by using the the `#` character in the optional-column.
		* **choice1, choice2, ..., choice14** (you can use these columns to quickly add choices. If you use many choices repeatedly or need more than 14 choices, it makes more sense to put them on the choices sheet)
* The second, optional sheet should have the name **choices**, if no such sheet exists, we use the second one.
	* The following column names are used (in order).
		* **list name** - you can reference the list of choices using this name on the 'survey' sheet.
		* **name** - the text for the choice that will be stored. This can only contain `a-zA-Z0-9_`.
		* **label** - the text for the choice that is displayed (if you leave this column out or empty, we'll use the name text)

#### Items
Surveys support the following item types. HTML5 form elements and validation are used and polyfilled where necessary using the [Webshims lib](http://afarkas.github.io/webshim/demos/index.html).

* `instruction` display text. instructions are displayed at least once and disappear only when there are no unanswered items left behind them (so putting an instruction directly before another ensures it will be displayed only once)
* `submit` display a submit button. No items are displayed after the submit button, until all of the ones preceding it have been answered. This is useful for pagination and to ensure that answers required for `skipif` or substitutions have been given. 
* multiple choice family
	* `mc` multipe choice (radio buttons), you can choose only one. Choices are (currently) defined using the choice1-12 columns
	* `mmc` multiple multiple choice (check boxes), you can choose several. Choices defined as above
	* `check` a single check box for confirmation of a statement.
	* `btnradio` like `mc` but radio buttons are styled as buttons containing the choice text
	* `btncheckbox` like `mmc` and `btnradio`
	* `btncheck` like `check` and `btnradio`
	* `btnrating min,max,step` This shows choice1 to the left, choice2 to the right and a series of buttons as defined by `min,max,step` in between. Defaults to `1,5,1`
	* `sex` shorthand for `btnradio` with the ♂, ♀ symbols as choices
	* `select` dropdowns, you can choose only one
	* `mselect` dropdowns, choose many
	* `select_add maxType` like `select`, allows users to choose an option not given. Uses [Select2](http://ivaynberg.github.io/select2/). `maxType` can be used to set an upper limit on the length of the user-added option. Defaults to 255.
	* `mselect_add maxType,maxChoose` like `mselect` and `select_add`, allows users to add options not given. `maxChoose` can be used to place an upper limit on the number of chooseable options.
	* `mc_heading` To get a tabular look, assign a constant width to the choices (with classes), give the heading the same choices as the `mc`s, and give the following `mc`s (or `mmc`s)  the same classes + hide_label. 
* `range min,max,step` these are sliders. The numeric value chosen is not displayed. Text to be shown to the left and right of the slider can be defined using the choice1 and choice2 fields. Defaults are `1,100,1`.
* `range_list min,max,step` like `range` but the individual steps are visually indicated using ticks and the chosen number is shown to the right. 
* `color` allows you to pick a color, using the OS color picker (or one polyfilled by Webshims)
* simple input family
	* `text` allows you to enter a text in a single-line input field. Adding a number `text 100` defines the maximum number of characters that may be entered.
	* `textarea` displays a multi-line input field
	* `letters` like `text`, allows only letters, not numbers.
	* `number min,max,step` for numbers. `step` defaults to `1`, using `any` will allow any decimals.
	* `email` for email addresses
	* `url`, `cc` (credit card number), `tel` are allowed too, but webshims doesn't polyfill them yet and most major browsers don't support them
* datetime family
	* `time` for time. a small clock is prepended to the input
	* `date` for a date
	* `datetime` for a date & time
	* `month`, `yearmonth` etc. should work too, but don't in most browsers
* `geolocation` shows a text input and a geolocation arrow. If the user allows it, the geolocation object is stored as a JSON string, if not the user can enter a text string.
* "server" family
	* `ip` saves the value of the PHP superglobal `$_SERVER['REMOTE_ADDR']` ie. the user's IP.
	* `referrer` saves the last outside referrer (if any), ie. how the user got to the site
	* `server var` saves the `$_SERVER` value with the index given by `var`. Can be used to store one of `'HTTP_USER_AGENT',	'HTTP_ACCEPT',	'HTTP_ACCEPT_CHARSET',	'HTTP_ACCEPT_ENCODING',	'HTTP_ACCEPT_LANGUAGE',	'HTTP_CONNECTION',	'HTTP_HOST',	'QUERY_STRING',	'REQUEST_TIME',	'REQUEST_TIME_FLOAT'`

#### skipif

You can make item display contingent on simple and complex conditions like `(survey1.married = 1) OR (survey2.in_relationship = 1 AND survey2.cohabit = 1)`.

## OpenCPU + R + Knitr + Markdown
In pauses, emails and pages you can display text to the user. This text is easily formatted using [Markdown](http://daringfireball.net/projects/markdown/) a simple syntax that formats text nicely if you simply write like you would write a plain text email. Markdown can be freely mixed with HTML, so you can e.g. insert icons from the [Font Awesome](http://fontawesome.io) library using `<i class="icon-smile"></i>`.

If you use knitr syntax, where Markdown can be used, the text will not just be parsed as Markdown (which is mostly static, unless you use Javascript), but also be parsed (anew each time) by [knitr](http://yihui.name/knitr/). Knitr allows for mixing R syntax chunks and Markdown. [R](http://www.r-project.org/) is a popular open-source statistical programming language, that you can use via [OpenCPU](https://public.opencpu.org/pages/), a RESTful interface to the language that deals with the problem that R was not meant to be used as part of web apps and is insecure. R data frames with the same names as the surveys they derive from will be available in this knitr call, they contain all data that the current user has filled out so far.  
Combined with norm data etc. you can tailor feedback to the user's input, e.g. show where the user lies on the bell curve etc.

### Example
This may sound complicated at first, but should be really simple to use, while still providing most of the functionality available to R users (e.g. pretty [ggplot2](http://ggplot2.org/) plots).

Using this syntax will yield the following results (assuming that the surveys "demographics" and "mood_diary" exist and contain the appropriate data)

	Hi `r demographics[1,]$first_name`!
	
	This is a graph showing how **your mood** fluctuated across the 50 days that you filled out our diary.
	
	### Graph
	```{r mood.plot}
	library(ggplot2)
	mood_diary$mood <- rowSums(user_data[,c('mood1','mood2','mood3')])
	qplot(Day, mood, data = mood_diary) + geom_smooth() + scale_y_continuous("Your mood") + theme_bw()
	```
	
#### This will be displayed to the user

Hi Petunia!

This is a graph showing how <strong>your mood</strong> fluctuated across the 50 days that you filled out our diary.

### Graph

(Image)


## Installation

If you want to test formr, you can simply clone this repository. 
You need to rename the "config_default" directory to "config" and supply your database settings. You will also have to make the folders "tmp" and "backups" writeable. 
If you get internal server errors, these most likely stem from .htaccess files or the aforementioned folder permissions.

## Problems and plans
### Security
#### Database
End user input should be safe, we use prepared queries etc.  

However, study creators can wreak havoc, because they can write SQL expressions for skipIfs, substitutions, branches, etc. 
A future version will rely on OpenCPU's R API to sandbox these problems.

#### API
* you can create run access tokens using the API
* you can end "External" units using the API

### Plans
* Todo: Should allow a "service message" to be displayed in runs, to make run completely inactive.
* Todo: Switch user-supplied run logic to R instead of SQL.
* Todo: Should be storing an API key hash, not the API key itself.


## Credit

### Author:
Ruben C. Arslan (2013)

### Based on work, ideas, and feedback by:
Linus Neumann, Jaap J. A. Denissen, Karsten Gebbert, Julia Zimmermann, Sarah Lennartz, Isabelle Habedank, Jörg Basedow

#### Funded by 

#### Friedrich-Schiller-Universität Jena
* Julia Zimmermann
* Franz J. Neyer

#### Humboldt Universität zu Berlin
Jaap J. A. Denissen